using System;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Microsoft.DirectX.DirectSound;


namespace SoundStream
{


    public class Voice 
    {
        private const int secondsOfPlay = 1;

        private WaveFormat waveFormat;     
        private SecondaryBuffer waveformBuffer;      
        private Device soundDevice;
        private short [] buffer;

        public delegate void PullAudioCallback(IntPtr data, int count);       


        // ******************************************************************************************
        public Voice(Control owner)
        {
            BufferDescription waveformBufferDescription;

            // Construct our own "virtual sound card".
            soundDevice = new Device();
            soundDevice.SetCooperativeLevel(owner, CooperativeLevel.Normal);

            // Define the format for the raw waveform data.
            waveFormat = new WaveFormat();
            waveFormat.FormatTag = WaveFormatTag.Pcm;
            waveFormat.SamplesPerSecond = 22050;
            waveFormat.BitsPerSample = 16;
            waveFormat.Channels = 1;

            waveFormat.BlockAlign = (short)(waveFormat.Channels * (waveFormat.BitsPerSample / 8));
            waveFormat.AverageBytesPerSecond = waveFormat.SamplesPerSecond * waveFormat.BlockAlign;

            // Describe the sound buffer we want to create below.
            waveformBufferDescription = new BufferDescription(waveFormat);
            waveformBufferDescription.BufferBytes = secondsOfPlay * waveFormat.AverageBytesPerSecond;
            waveformBufferDescription.ControlVolume = true;
            waveformBufferDescription.GlobalFocus = true;

            // Create the actual sound buffer; the heart of the streaming sound features.
            waveformBuffer = new SecondaryBuffer(waveformBufferDescription, soundDevice);
        }

        // ******************************************************************************************
        public void stop()
        {
            if (waveformBuffer != null)
            {
                waveformBuffer.Stop();               
            }
        }

        // ******************************************************************************************
        public void start()
        {
            if (this.waveformBuffer.Status.Playing != true)
            {
                waveformBuffer.SetCurrentPosition(0);
                waveformBuffer.Play(0, BufferPlayFlags.Looping);
            }
        }

        // ******************************************************************************************
        public void init(float freq,int volume)
        {
            int length = (int)(waveFormat.SamplesPerSecond);
            this.buffer = new short[length];
            this.Generate_Tone(this.buffer, length, volume, freq);

            waveformBuffer.Write(0, this.buffer, LockFlag.EntireBuffer);
        }

        // ******************************************************************************************
        private void Generate_Tone(short[] mixBuffer, int samples, int maxVolume,float frequency)
        {
            int i;            
            float T = samples / frequency;
            float factor = (float)(2 * Math.PI) / T;

            for (i = 0; i < samples; i++)
            {
                mixBuffer[i] = (short)(maxVolume * Math.Sin((float)i*factor));
            }
        }

    }
}   